# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation-init.j2

"""Auto-generated package that contains models used by the AccelByte Gaming Services Platform Service."""

__version__ = "6.0.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .mock_play_station_stream_event import MockPlayStationStreamEvent
from .mock_xbl_clawback_event import MockXblClawbackEvent
from .query_iap_clawback_history import QueryIAPClawbackHistory
from .query_iap_clawback_history import (
    EventTypeEnum as QueryIAPClawbackHistoryEventTypeEnum,
    StatusEnum as QueryIAPClawbackHistoryStatusEnum,
)



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum


class AllowedPlatformOriginsEnum(StrEnum):
    EPIC = "Epic"
    GOOGLEPLAY = "GooglePlay"
    IOS = "IOS"
    NINTENDO = "Nintendo"
    OCULUS = "Oculus"
    OTHER = "Other"
    PLAYSTATION = "Playstation"
    STEAM = "Steam"
    SYSTEM = "System"
    TWITCH = "Twitch"
    XBOX = "Xbox"


class EntitlementPlatformConfigUpdate(Model):
    """A DTO object for updating entitlement platform config API call. (EntitlementPlatformConfigUpdate)

    Properties:
        allowed_platform_origins: (allowedPlatformOrigins) OPTIONAL Union[List[str], List[AllowedPlatformOriginsEnum]]
    """

    # region fields

    allowed_platform_origins: Union[
        List[str], List[AllowedPlatformOriginsEnum]
    ]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_allowed_platform_origins(
        self, value: Union[List[str], List[AllowedPlatformOriginsEnum]]
    ) -> EntitlementPlatformConfigUpdate:
        self.allowed_platform_origins = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "allowed_platform_origins"):
            result["allowedPlatformOrigins"] = [
                str(i0) for i0 in self.allowed_platform_origins
            ]
        elif include_empty:
            result["allowedPlatformOrigins"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        allowed_platform_origins: Optional[
            Union[List[str], List[AllowedPlatformOriginsEnum]]
        ] = None,
        **kwargs,
    ) -> EntitlementPlatformConfigUpdate:
        instance = cls()
        if allowed_platform_origins is not None:
            instance.allowed_platform_origins = allowed_platform_origins
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> EntitlementPlatformConfigUpdate:
        instance = cls()
        if not dict_:
            return instance
        if (
            "allowedPlatformOrigins" in dict_
            and dict_["allowedPlatformOrigins"] is not None
        ):
            instance.allowed_platform_origins = [
                str(i0) for i0 in dict_["allowedPlatformOrigins"]
            ]
        elif include_empty:
            instance.allowed_platform_origins = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, EntitlementPlatformConfigUpdate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[EntitlementPlatformConfigUpdate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        EntitlementPlatformConfigUpdate,
        List[EntitlementPlatformConfigUpdate],
        Dict[Any, EntitlementPlatformConfigUpdate],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "allowedPlatformOrigins": "allowed_platform_origins",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "allowedPlatformOrigins": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "allowedPlatformOrigins": [
                "Epic",
                "GooglePlay",
                "IOS",
                "Nintendo",
                "Oculus",
                "Other",
                "Playstation",
                "Steam",
                "System",
                "Twitch",
                "Xbox",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import same_doc_as

from ..models import ErrorEntity
from ..models import FullViewInfo
from ..models import ListViewInfo
from ..models import ValidationErrorEntity
from ..models import ViewCreate
from ..models import ViewInfo
from ..models import ViewUpdate

from ..operations.view import CreateView
from ..operations.view import DeleteView
from ..operations.view import GetView
from ..operations.view import ListViews
from ..operations.view import PublicListViews
from ..operations.view import UpdateView


@same_doc_as(CreateView)
def create_view(
    body: ViewCreate,
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a view (createView)

    This API is used to create a view.

    Other detail info:

      * Returns : created a view



    ## Restrictions for localization extension


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Properties:
        url: /platform/admin/namespaces/{namespace}/views

        method: POST

        tags: ["View"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ViewCreate in body

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        201: Created - FullViewInfo (successful operation)

        400: Bad Request - ErrorEntity (30021: Default language [{language}] required)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateView.create(
        body=body,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateView)
async def create_view_async(
    body: ViewCreate,
    store_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a view (createView)

    This API is used to create a view.

    Other detail info:

      * Returns : created a view



    ## Restrictions for localization extension


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Properties:
        url: /platform/admin/namespaces/{namespace}/views

        method: POST

        tags: ["View"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ViewCreate in body

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        201: Created - FullViewInfo (successful operation)

        400: Bad Request - ErrorEntity (30021: Default language [{language}] required)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateView.create(
        body=body,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteView)
def delete_view(
    store_id: str,
    view_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a view (deleteView)

    This API is used to delete a view.It will also delete all the related sections

    Properties:
        url: /platform/admin/namespaces/{namespace}/views/{viewId}

        method: DELETE

        tags: ["View"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        view_id: (viewId) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        204: No Content - (Delete view successfully)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteView.create(
        store_id=store_id,
        view_id=view_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteView)
async def delete_view_async(
    store_id: str,
    view_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a view (deleteView)

    This API is used to delete a view.It will also delete all the related sections

    Properties:
        url: /platform/admin/namespaces/{namespace}/views/{viewId}

        method: DELETE

        tags: ["View"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        view_id: (viewId) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        204: No Content - (Delete view successfully)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteView.create(
        store_id=store_id,
        view_id=view_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetView)
def get_view(
    view_id: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a view (getView)

    This API is used to get a view.

    Other detail info:

      * Returns : view data

    Properties:
        url: /platform/admin/namespaces/{namespace}/views/{viewId}

        method: GET

        tags: ["View"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        view_id: (viewId) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - FullViewInfo (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetView.create(
        view_id=view_id,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetView)
async def get_view_async(
    view_id: str,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a view (getView)

    This API is used to get a view.

    Other detail info:

      * Returns : view data

    Properties:
        url: /platform/admin/namespaces/{namespace}/views/{viewId}

        method: GET

        tags: ["View"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        view_id: (viewId) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - FullViewInfo (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}])
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetView.create(
        view_id=view_id,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListViews)
def list_views(
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all views (listViews)

    This API is used to list all views.

    Other detail info:

      * Returns : the list of views

    Properties:
        url: /platform/admin/namespaces/{namespace}/views

        method: GET

        tags: ["View"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[ListViewInfo] (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListViews.create(
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListViews)
async def list_views_async(
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all views (listViews)

    This API is used to list all views.

    Other detail info:

      * Returns : the list of views

    Properties:
        url: /platform/admin/namespaces/{namespace}/views

        method: GET

        tags: ["View"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[ListViewInfo] (successful operation)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30142: Published store does not exist in namespace [{namespace}])

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = ListViews.create(
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicListViews)
def public_list_views(
    user_id: str,
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get all views (publicListViews)

    This API is used to get all views.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store views)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store views)
      *  Returns : all views

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/views

        method: GET

        tags: ["View"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[ViewInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListViews.create(
        user_id=user_id,
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicListViews)
async def public_list_views_async(
    user_id: str,
    language: Optional[str] = None,
    store_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get all views (publicListViews)

    This API is used to get all views.

    Other detail info:

      * Optional permission : resource="PREVIEW", action=1(CREATE) (user with this permission can view draft store views)
      *  Optional permission : resource="SANDBOX", action=1(CREATE) (user with this permission can view draft store views)
      *  Returns : all views

    Properties:
        url: /platform/public/namespaces/{namespace}/users/{userId}/views

        method: GET

        tags: ["View"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

        language: (language) OPTIONAL str in query

        store_id: (storeId) OPTIONAL str in query

    Responses:
        200: OK - List[ViewInfo] (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = PublicListViews.create(
        user_id=user_id,
        language=language,
        store_id=store_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateView)
def update_view(
    body: ViewUpdate,
    store_id: str,
    view_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a view (updateView)

    This API is used to update a view.

    Other detail info:

      * Returns : updated view data



    ## Restrictions for localization extension


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Properties:
        url: /platform/admin/namespaces/{namespace}/views/{viewId}

        method: PUT

        tags: ["View"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ViewUpdate in body

        namespace: (namespace) REQUIRED str in path

        view_id: (viewId) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        200: OK - FullViewInfo (successful operation)

        400: Bad Request - ErrorEntity (30021: Default language [{language}] required)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateView.create(
        body=body,
        store_id=store_id,
        view_id=view_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateView)
async def update_view_async(
    body: ViewUpdate,
    store_id: str,
    view_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update a view (updateView)

    This API is used to update a view.

    Other detail info:

      * Returns : updated view data



    ## Restrictions for localization extension


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Properties:
        url: /platform/admin/namespaces/{namespace}/views/{viewId}

        method: PUT

        tags: ["View"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ViewUpdate in body

        namespace: (namespace) REQUIRED str in path

        view_id: (viewId) REQUIRED str in path

        store_id: (storeId) REQUIRED str in query

    Responses:
        200: OK - FullViewInfo (successful operation)

        400: Bad Request - ErrorEntity (30021: Default language [{language}] required)

        404: Not Found - ErrorEntity (30141: Store [{storeId}] does not exist in namespace [{namespace}] | 30641: View [{viewId}] does not exist in namespace [{namespace}])

        409: Conflict - ErrorEntity (30173: Published store can't modify content)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateView.create(
        body=body,
        store_id=store_id,
        view_id=view_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class GoogleIAPConfigRequest(Model):
    """Google IAP config request (GoogleIAPConfigRequest)

    Properties:
        application_name: (applicationName) REQUIRED str

        package_name: (packageName) REQUIRED str

        service_account_id: (serviceAccountId) REQUIRED str

        notification_token_audience: (notificationTokenAudience) OPTIONAL str

        notification_token_email: (notificationTokenEmail) OPTIONAL str
    """

    # region fields

    application_name: str  # REQUIRED
    package_name: str  # REQUIRED
    service_account_id: str  # REQUIRED
    notification_token_audience: str  # OPTIONAL
    notification_token_email: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_application_name(self, value: str) -> GoogleIAPConfigRequest:
        self.application_name = value
        return self

    def with_package_name(self, value: str) -> GoogleIAPConfigRequest:
        self.package_name = value
        return self

    def with_service_account_id(self, value: str) -> GoogleIAPConfigRequest:
        self.service_account_id = value
        return self

    def with_notification_token_audience(self, value: str) -> GoogleIAPConfigRequest:
        self.notification_token_audience = value
        return self

    def with_notification_token_email(self, value: str) -> GoogleIAPConfigRequest:
        self.notification_token_email = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "application_name"):
            result["applicationName"] = str(self.application_name)
        elif include_empty:
            result["applicationName"] = ""
        if hasattr(self, "package_name"):
            result["packageName"] = str(self.package_name)
        elif include_empty:
            result["packageName"] = ""
        if hasattr(self, "service_account_id"):
            result["serviceAccountId"] = str(self.service_account_id)
        elif include_empty:
            result["serviceAccountId"] = ""
        if hasattr(self, "notification_token_audience"):
            result["notificationTokenAudience"] = str(self.notification_token_audience)
        elif include_empty:
            result["notificationTokenAudience"] = ""
        if hasattr(self, "notification_token_email"):
            result["notificationTokenEmail"] = str(self.notification_token_email)
        elif include_empty:
            result["notificationTokenEmail"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        application_name: str,
        package_name: str,
        service_account_id: str,
        notification_token_audience: Optional[str] = None,
        notification_token_email: Optional[str] = None,
        **kwargs,
    ) -> GoogleIAPConfigRequest:
        instance = cls()
        instance.application_name = application_name
        instance.package_name = package_name
        instance.service_account_id = service_account_id
        if notification_token_audience is not None:
            instance.notification_token_audience = notification_token_audience
        if notification_token_email is not None:
            instance.notification_token_email = notification_token_email
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> GoogleIAPConfigRequest:
        instance = cls()
        if not dict_:
            return instance
        if "applicationName" in dict_ and dict_["applicationName"] is not None:
            instance.application_name = str(dict_["applicationName"])
        elif include_empty:
            instance.application_name = ""
        if "packageName" in dict_ and dict_["packageName"] is not None:
            instance.package_name = str(dict_["packageName"])
        elif include_empty:
            instance.package_name = ""
        if "serviceAccountId" in dict_ and dict_["serviceAccountId"] is not None:
            instance.service_account_id = str(dict_["serviceAccountId"])
        elif include_empty:
            instance.service_account_id = ""
        if (
            "notificationTokenAudience" in dict_
            and dict_["notificationTokenAudience"] is not None
        ):
            instance.notification_token_audience = str(
                dict_["notificationTokenAudience"]
            )
        elif include_empty:
            instance.notification_token_audience = ""
        if (
            "notificationTokenEmail" in dict_
            and dict_["notificationTokenEmail"] is not None
        ):
            instance.notification_token_email = str(dict_["notificationTokenEmail"])
        elif include_empty:
            instance.notification_token_email = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, GoogleIAPConfigRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[GoogleIAPConfigRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        GoogleIAPConfigRequest,
        List[GoogleIAPConfigRequest],
        Dict[Any, GoogleIAPConfigRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "applicationName": "application_name",
            "packageName": "package_name",
            "serviceAccountId": "service_account_id",
            "notificationTokenAudience": "notification_token_audience",
            "notificationTokenEmail": "notification_token_email",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "applicationName": True,
            "packageName": True,
            "serviceAccountId": True,
            "notificationTokenAudience": False,
            "notificationTokenEmail": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum

from ..models.fulfillment_action_history import FulfillmentActionHistory
from ..models.fulfillment_item import FulfillmentItem
from ..models.fulfillment_state_info import FulfillmentStateInfo


class StateEnum(StrEnum):
    FULFILLED = "FULFILLED"
    FULFILL_FAILED = "FULFILL_FAILED"
    REVOKED = "REVOKED"
    REVOKE_FAILED = "REVOKE_FAILED"


class FulfillmentInfo(Model):
    """Fulfillment info (FulfillmentInfo)

    Properties:
        created_at: (createdAt) REQUIRED str

        id_: (id) REQUIRED str

        namespace: (namespace) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        user_id: (userId) REQUIRED str

        actions: (actions) OPTIONAL List[FulfillmentActionHistory]

        items: (items) OPTIONAL List[FulfillmentItem]

        state: (state) OPTIONAL Union[str, StateEnum]

        state_info: (stateInfo) OPTIONAL FulfillmentStateInfo

        transaction_id: (transactionId) OPTIONAL str
    """

    # region fields

    created_at: str  # REQUIRED
    id_: str  # REQUIRED
    namespace: str  # REQUIRED
    updated_at: str  # REQUIRED
    user_id: str  # REQUIRED
    actions: List[FulfillmentActionHistory]  # OPTIONAL
    items: List[FulfillmentItem]  # OPTIONAL
    state: Union[str, StateEnum]  # OPTIONAL
    state_info: FulfillmentStateInfo  # OPTIONAL
    transaction_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> FulfillmentInfo:
        self.created_at = value
        return self

    def with_id(self, value: str) -> FulfillmentInfo:
        self.id_ = value
        return self

    def with_namespace(self, value: str) -> FulfillmentInfo:
        self.namespace = value
        return self

    def with_updated_at(self, value: str) -> FulfillmentInfo:
        self.updated_at = value
        return self

    def with_user_id(self, value: str) -> FulfillmentInfo:
        self.user_id = value
        return self

    def with_actions(self, value: List[FulfillmentActionHistory]) -> FulfillmentInfo:
        self.actions = value
        return self

    def with_items(self, value: List[FulfillmentItem]) -> FulfillmentInfo:
        self.items = value
        return self

    def with_state(self, value: Union[str, StateEnum]) -> FulfillmentInfo:
        self.state = value
        return self

    def with_state_info(self, value: FulfillmentStateInfo) -> FulfillmentInfo:
        self.state_info = value
        return self

    def with_transaction_id(self, value: str) -> FulfillmentInfo:
        self.transaction_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "actions"):
            result["actions"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.actions
            ]
        elif include_empty:
            result["actions"] = []
        if hasattr(self, "items"):
            result["items"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.items
            ]
        elif include_empty:
            result["items"] = []
        if hasattr(self, "state"):
            result["state"] = str(self.state)
        elif include_empty:
            result["state"] = Union[str, StateEnum]()
        if hasattr(self, "state_info"):
            result["stateInfo"] = self.state_info.to_dict(include_empty=include_empty)
        elif include_empty:
            result["stateInfo"] = FulfillmentStateInfo()
        if hasattr(self, "transaction_id"):
            result["transactionId"] = str(self.transaction_id)
        elif include_empty:
            result["transactionId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        id_: str,
        namespace: str,
        updated_at: str,
        user_id: str,
        actions: Optional[List[FulfillmentActionHistory]] = None,
        items: Optional[List[FulfillmentItem]] = None,
        state: Optional[Union[str, StateEnum]] = None,
        state_info: Optional[FulfillmentStateInfo] = None,
        transaction_id: Optional[str] = None,
        **kwargs,
    ) -> FulfillmentInfo:
        instance = cls()
        instance.created_at = created_at
        instance.id_ = id_
        instance.namespace = namespace
        instance.updated_at = updated_at
        instance.user_id = user_id
        if actions is not None:
            instance.actions = actions
        if items is not None:
            instance.items = items
        if state is not None:
            instance.state = state
        if state_info is not None:
            instance.state_info = state_info
        if transaction_id is not None:
            instance.transaction_id = transaction_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> FulfillmentInfo:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "actions" in dict_ and dict_["actions"] is not None:
            instance.actions = [
                FulfillmentActionHistory.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["actions"]
            ]
        elif include_empty:
            instance.actions = []
        if "items" in dict_ and dict_["items"] is not None:
            instance.items = [
                FulfillmentItem.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["items"]
            ]
        elif include_empty:
            instance.items = []
        if "state" in dict_ and dict_["state"] is not None:
            instance.state = str(dict_["state"])
        elif include_empty:
            instance.state = Union[str, StateEnum]()
        if "stateInfo" in dict_ and dict_["stateInfo"] is not None:
            instance.state_info = FulfillmentStateInfo.create_from_dict(
                dict_["stateInfo"], include_empty=include_empty
            )
        elif include_empty:
            instance.state_info = FulfillmentStateInfo()
        if "transactionId" in dict_ and dict_["transactionId"] is not None:
            instance.transaction_id = str(dict_["transactionId"])
        elif include_empty:
            instance.transaction_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, FulfillmentInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[FulfillmentInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[FulfillmentInfo, List[FulfillmentInfo], Dict[Any, FulfillmentInfo]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "id": "id_",
            "namespace": "namespace",
            "updatedAt": "updated_at",
            "userId": "user_id",
            "actions": "actions",
            "items": "items",
            "state": "state",
            "stateInfo": "state_info",
            "transactionId": "transaction_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "id": True,
            "namespace": True,
            "updatedAt": True,
            "userId": True,
            "actions": False,
            "items": False,
            "state": False,
            "stateInfo": False,
            "transactionId": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "state": ["FULFILLED", "FULFILL_FAILED", "REVOKED", "REVOKE_FAILED"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import same_doc_as

from ..models import TradeActionPagingSlicedResult
from ..models import TradeChainActionHistoryInfo
from ..models import TradeChainedActionCommitRequest

from ..operations.trade_action import Commit
from ..operations.trade_action import GetTradeHistoryByCriteria
from ..operations.trade_action import (
    GetTradeHistoryByCriteriaStatusEnum,
)
from ..operations.trade_action import GetTradeHistoryByTransactionId
from ..models import TradeChainActionHistoryInfoStatusEnum


@same_doc_as(Commit)
def commit(
    body: TradeChainedActionCommitRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """commit a chain of actions or operations (commit)

    This API is used to create a chained operations

    Other detail info:

      * Returns : chain action history
      *  FULFILL_ITEM operation supported item type : INGAMEITEM,LOOTBOX,OPTIONBOX



    ## Restrictions for metadata


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Properties:
        url: /platform/admin/namespaces/{namespace}/trade/commit

        method: POST

        tags: ["TradeAction"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED TradeChainedActionCommitRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - TradeChainActionHistoryInfo (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Commit.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(Commit)
async def commit_async(
    body: TradeChainedActionCommitRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """commit a chain of actions or operations (commit)

    This API is used to create a chained operations

    Other detail info:

      * Returns : chain action history
      *  FULFILL_ITEM operation supported item type : INGAMEITEM,LOOTBOX,OPTIONBOX



    ## Restrictions for metadata


    1. Cannot use "." as the key name
    -


        { "data.2": "value" }


    2. Cannot use "$" as the prefix in key names
    -


        { "$data": "value" }

    Properties:
        url: /platform/admin/namespaces/{namespace}/trade/commit

        method: POST

        tags: ["TradeAction"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED TradeChainedActionCommitRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - TradeChainActionHistoryInfo (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = Commit.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetTradeHistoryByCriteria)
def get_trade_history_by_criteria(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    status: Optional[Union[str, GetTradeHistoryByCriteriaStatusEnum]] = None,
    type_: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve trade history by criteria (getTradeHistoryByCriteria)

    This API is used to fetch trade history based on the provided criteria

    Other detail info:

      * Returns : trade history list based on criteria

    Properties:
        url: /platform/admin/namespaces/{namespace}/trade/history/byCriteria

        method: GET

        tags: ["TradeAction"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

        type_: (type) OPTIONAL str in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - TradeActionPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetTradeHistoryByCriteria.create(
        limit=limit,
        offset=offset,
        status=status,
        type_=type_,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetTradeHistoryByCriteria)
async def get_trade_history_by_criteria_async(
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    status: Optional[Union[str, GetTradeHistoryByCriteriaStatusEnum]] = None,
    type_: Optional[str] = None,
    user_id: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve trade history by criteria (getTradeHistoryByCriteria)

    This API is used to fetch trade history based on the provided criteria

    Other detail info:

      * Returns : trade history list based on criteria

    Properties:
        url: /platform/admin/namespaces/{namespace}/trade/history/byCriteria

        method: GET

        tags: ["TradeAction"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

        status: (status) OPTIONAL Union[str, StatusEnum] in query

        type_: (type) OPTIONAL str in query

        user_id: (userId) OPTIONAL str in query

    Responses:
        200: OK - TradeActionPagingSlicedResult (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetTradeHistoryByCriteria.create(
        limit=limit,
        offset=offset,
        status=status,
        type_=type_,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetTradeHistoryByTransactionId)
def get_trade_history_by_transaction_id(
    transaction_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve trade history by transaction ID (getTradeHistoryByTransactionId)

    This API is used to fetch a specific trade history using transaction ID

    Other detail info:

      * Returns : trade history based on transaction ID

    Properties:
        url: /platform/admin/namespaces/{namespace}/trade/{transactionId}

        method: GET

        tags: ["TradeAction"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

    Responses:
        200: OK - TradeChainActionHistoryInfo (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetTradeHistoryByTransactionId.create(
        transaction_id=transaction_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetTradeHistoryByTransactionId)
async def get_trade_history_by_transaction_id_async(
    transaction_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve trade history by transaction ID (getTradeHistoryByTransactionId)

    This API is used to fetch a specific trade history using transaction ID

    Other detail info:

      * Returns : trade history based on transaction ID

    Properties:
        url: /platform/admin/namespaces/{namespace}/trade/{transactionId}

        method: GET

        tags: ["TradeAction"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        transaction_id: (transactionId) REQUIRED str in path

    Responses:
        200: OK - TradeChainActionHistoryInfo (successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetTradeHistoryByTransactionId.create(
        transaction_id=transaction_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model

from ..models.xbl_clawback_contract_v2 import XblClawbackContractV2


class XblClawbackEvent(Model):
    """Xbl clawback event (XblClawbackEvent)

    Properties:
        data: (data) REQUIRED XblClawbackContractV2

        id_: (id) REQUIRED str

        source: (source) REQUIRED str

        type_: (type) REQUIRED str

        datacontenttype: (datacontenttype) OPTIONAL str

        spec_version: (specVersion) OPTIONAL str

        subject: (subject) OPTIONAL str

        time: (time) OPTIONAL str

        traceparent: (traceparent) OPTIONAL str
    """

    # region fields

    data: XblClawbackContractV2  # REQUIRED
    id_: str  # REQUIRED
    source: str  # REQUIRED
    type_: str  # REQUIRED
    datacontenttype: str  # OPTIONAL
    spec_version: str  # OPTIONAL
    subject: str  # OPTIONAL
    time: str  # OPTIONAL
    traceparent: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_data(self, value: XblClawbackContractV2) -> XblClawbackEvent:
        self.data = value
        return self

    def with_id(self, value: str) -> XblClawbackEvent:
        self.id_ = value
        return self

    def with_source(self, value: str) -> XblClawbackEvent:
        self.source = value
        return self

    def with_type(self, value: str) -> XblClawbackEvent:
        self.type_ = value
        return self

    def with_datacontenttype(self, value: str) -> XblClawbackEvent:
        self.datacontenttype = value
        return self

    def with_spec_version(self, value: str) -> XblClawbackEvent:
        self.spec_version = value
        return self

    def with_subject(self, value: str) -> XblClawbackEvent:
        self.subject = value
        return self

    def with_time(self, value: str) -> XblClawbackEvent:
        self.time = value
        return self

    def with_traceparent(self, value: str) -> XblClawbackEvent:
        self.traceparent = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "data"):
            result["data"] = self.data.to_dict(include_empty=include_empty)
        elif include_empty:
            result["data"] = XblClawbackContractV2()
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "source"):
            result["source"] = str(self.source)
        elif include_empty:
            result["source"] = ""
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        if hasattr(self, "datacontenttype"):
            result["datacontenttype"] = str(self.datacontenttype)
        elif include_empty:
            result["datacontenttype"] = ""
        if hasattr(self, "spec_version"):
            result["specVersion"] = str(self.spec_version)
        elif include_empty:
            result["specVersion"] = ""
        if hasattr(self, "subject"):
            result["subject"] = str(self.subject)
        elif include_empty:
            result["subject"] = ""
        if hasattr(self, "time"):
            result["time"] = str(self.time)
        elif include_empty:
            result["time"] = ""
        if hasattr(self, "traceparent"):
            result["traceparent"] = str(self.traceparent)
        elif include_empty:
            result["traceparent"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        data: XblClawbackContractV2,
        id_: str,
        source: str,
        type_: str,
        datacontenttype: Optional[str] = None,
        spec_version: Optional[str] = None,
        subject: Optional[str] = None,
        time: Optional[str] = None,
        traceparent: Optional[str] = None,
        **kwargs,
    ) -> XblClawbackEvent:
        instance = cls()
        instance.data = data
        instance.id_ = id_
        instance.source = source
        instance.type_ = type_
        if datacontenttype is not None:
            instance.datacontenttype = datacontenttype
        if spec_version is not None:
            instance.spec_version = spec_version
        if subject is not None:
            instance.subject = subject
        if time is not None:
            instance.time = time
        if traceparent is not None:
            instance.traceparent = traceparent
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> XblClawbackEvent:
        instance = cls()
        if not dict_:
            return instance
        if "data" in dict_ and dict_["data"] is not None:
            instance.data = XblClawbackContractV2.create_from_dict(
                dict_["data"], include_empty=include_empty
            )
        elif include_empty:
            instance.data = XblClawbackContractV2()
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "source" in dict_ and dict_["source"] is not None:
            instance.source = str(dict_["source"])
        elif include_empty:
            instance.source = ""
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        if "datacontenttype" in dict_ and dict_["datacontenttype"] is not None:
            instance.datacontenttype = str(dict_["datacontenttype"])
        elif include_empty:
            instance.datacontenttype = ""
        if "specVersion" in dict_ and dict_["specVersion"] is not None:
            instance.spec_version = str(dict_["specVersion"])
        elif include_empty:
            instance.spec_version = ""
        if "subject" in dict_ and dict_["subject"] is not None:
            instance.subject = str(dict_["subject"])
        elif include_empty:
            instance.subject = ""
        if "time" in dict_ and dict_["time"] is not None:
            instance.time = str(dict_["time"])
        elif include_empty:
            instance.time = ""
        if "traceparent" in dict_ and dict_["traceparent"] is not None:
            instance.traceparent = str(dict_["traceparent"])
        elif include_empty:
            instance.traceparent = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, XblClawbackEvent]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[XblClawbackEvent]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[XblClawbackEvent, List[XblClawbackEvent], Dict[Any, XblClawbackEvent]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "data": "data",
            "id": "id_",
            "source": "source",
            "type": "type_",
            "datacontenttype": "datacontenttype",
            "specVersion": "spec_version",
            "subject": "subject",
            "time": "time",
            "traceparent": "traceparent",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "data": True,
            "id": True,
            "source": True,
            "type": True,
            "datacontenttype": False,
            "specVersion": False,
            "subject": False,
            "time": False,
            "traceparent": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# AccelByte Gaming Services Platform Service

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Operation
from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import HttpResponse
from accelbyte_py_sdk.core import StrEnum

from ...models import SimpleUserDLCRewardContentsResponse


class TypeEnum(StrEnum):
    EPICGAMES = "EPICGAMES"
    OCULUS = "OCULUS"
    PSN = "PSN"
    STEAM = "STEAM"
    XBOX = "XBOX"


class PublicGetMyDLCContent(Operation):
    """Get user dlc reward contents (publicGetMyDLCContent)

    Get user dlc reward contents. If includeAllNamespaces is false, will only return the dlc synced from the current namespace
    Other detail info:

      * Returns : user dlc

    Properties:
        url: /platform/public/users/me/dlc/content

        method: GET

        tags: ["DLC"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        include_all_namespaces: (includeAllNamespaces) OPTIONAL bool in query

        type_: (type) REQUIRED Union[str, TypeEnum] in query

    Responses:
        200: OK - SimpleUserDLCRewardContentsResponse (successful operation)
    """

    # region fields

    _url: str = "/platform/public/users/me/dlc/content"
    _path: str = "/platform/public/users/me/dlc/content"
    _base_path: str = ""
    _method: str = "GET"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["BEARER_AUTH"]]
    _location_query: str = None

    service_name: Optional[str] = "platform"

    include_all_namespaces: bool  # OPTIONAL in [query]
    type_: Union[str, TypeEnum]  # REQUIRED in [query]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def path(self) -> str:
        return self._path

    @property
    def base_path(self) -> str:
        return self._base_path

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "query": self.get_query_params(),
        }

    def get_query_params(self) -> dict:
        result = {}
        if hasattr(self, "include_all_namespaces"):
            result["includeAllNamespaces"] = self.include_all_namespaces
        if hasattr(self, "type_"):
            result["type"] = self.type_
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_include_all_namespaces(self, value: bool) -> PublicGetMyDLCContent:
        self.include_all_namespaces = value
        return self

    def with_type(self, value: Union[str, TypeEnum]) -> PublicGetMyDLCContent:
        self.type_ = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "include_all_namespaces") and self.include_all_namespaces:
            result["includeAllNamespaces"] = bool(self.include_all_namespaces)
        elif include_empty:
            result["includeAllNamespaces"] = False
        if hasattr(self, "type_") and self.type_:
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = Union[str, TypeEnum]()
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, SimpleUserDLCRewardContentsResponse], Union[None, HttpResponse]
    ]:
        """Parse the given response.

        200: OK - SimpleUserDLCRewardContentsResponse (successful operation)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return SimpleUserDLCRewardContentsResponse.create_from_dict(content), None

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        type_: Union[str, TypeEnum],
        include_all_namespaces: Optional[bool] = None,
        **kwargs,
    ) -> PublicGetMyDLCContent:
        instance = cls()
        instance.type_ = type_
        if include_all_namespaces is not None:
            instance.include_all_namespaces = include_all_namespaces
        if x_flight_id := kwargs.get("x_flight_id", None):
            instance.x_flight_id = x_flight_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PublicGetMyDLCContent:
        instance = cls()
        if (
            "includeAllNamespaces" in dict_
            and dict_["includeAllNamespaces"] is not None
        ):
            instance.include_all_namespaces = bool(dict_["includeAllNamespaces"])
        elif include_empty:
            instance.include_all_namespaces = False
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = Union[str, TypeEnum]()
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "includeAllNamespaces": "include_all_namespaces",
            "type": "type_",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "includeAllNamespaces": False,
            "type": True,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "type": ["EPICGAMES", "OCULUS", "PSN", "STEAM", "XBOX"],  # in query
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum

from ..models.credit_summary import CreditSummary
from ..models.entitlement_summary import EntitlementSummary
from ..models.extension_fulfillment_summary import ExtensionFulfillmentSummary
from ..models.fulfillment_error import FulfillmentError
from ..models.fulfillment_item import FulfillmentItem


class StatusEnum(StrEnum):
    FAIL = "FAIL"
    SUCCESS = "SUCCESS"


class EntitlementOriginEnum(StrEnum):
    EPIC = "Epic"
    GOOGLEPLAY = "GooglePlay"
    IOS = "IOS"
    NINTENDO = "Nintendo"
    OCULUS = "Oculus"
    OTHER = "Other"
    PLAYSTATION = "Playstation"
    STEAM = "Steam"
    SYSTEM = "System"
    TWITCH = "Twitch"
    XBOX = "Xbox"


class FulfillmentHistoryInfo(Model):
    """Fulfillment history info (FulfillmentHistoryInfo)

    Properties:
        created_at: (createdAt) REQUIRED str

        id_: (id) REQUIRED str

        namespace: (namespace) REQUIRED str

        status: (status) REQUIRED Union[str, StatusEnum]

        updated_at: (updatedAt) REQUIRED str

        user_id: (userId) REQUIRED str

        code: (code) OPTIONAL str

        credit_summaries: (creditSummaries) OPTIONAL List[CreditSummary]

        entitlement_origin: (entitlementOrigin) OPTIONAL Union[str, EntitlementOriginEnum]

        entitlement_summaries: (entitlementSummaries) OPTIONAL List[EntitlementSummary]

        extension_fulfillment_summaries: (extensionFulfillmentSummaries) OPTIONAL List[ExtensionFulfillmentSummary]

        fulfill_items: (fulfillItems) OPTIONAL List[FulfillmentItem]

        fulfillment_error: (fulfillmentError) OPTIONAL FulfillmentError

        granted_item_ids: (grantedItemIds) OPTIONAL List[str]

        order_no: (orderNo) OPTIONAL str
    """

    # region fields

    created_at: str  # REQUIRED
    id_: str  # REQUIRED
    namespace: str  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    updated_at: str  # REQUIRED
    user_id: str  # REQUIRED
    code: str  # OPTIONAL
    credit_summaries: List[CreditSummary]  # OPTIONAL
    entitlement_origin: Union[str, EntitlementOriginEnum]  # OPTIONAL
    entitlement_summaries: List[EntitlementSummary]  # OPTIONAL
    extension_fulfillment_summaries: List[ExtensionFulfillmentSummary]  # OPTIONAL
    fulfill_items: List[FulfillmentItem]  # OPTIONAL
    fulfillment_error: FulfillmentError  # OPTIONAL
    granted_item_ids: List[str]  # OPTIONAL
    order_no: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_created_at(self, value: str) -> FulfillmentHistoryInfo:
        self.created_at = value
        return self

    def with_id(self, value: str) -> FulfillmentHistoryInfo:
        self.id_ = value
        return self

    def with_namespace(self, value: str) -> FulfillmentHistoryInfo:
        self.namespace = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> FulfillmentHistoryInfo:
        self.status = value
        return self

    def with_updated_at(self, value: str) -> FulfillmentHistoryInfo:
        self.updated_at = value
        return self

    def with_user_id(self, value: str) -> FulfillmentHistoryInfo:
        self.user_id = value
        return self

    def with_code(self, value: str) -> FulfillmentHistoryInfo:
        self.code = value
        return self

    def with_credit_summaries(
        self, value: List[CreditSummary]
    ) -> FulfillmentHistoryInfo:
        self.credit_summaries = value
        return self

    def with_entitlement_origin(
        self, value: Union[str, EntitlementOriginEnum]
    ) -> FulfillmentHistoryInfo:
        self.entitlement_origin = value
        return self

    def with_entitlement_summaries(
        self, value: List[EntitlementSummary]
    ) -> FulfillmentHistoryInfo:
        self.entitlement_summaries = value
        return self

    def with_extension_fulfillment_summaries(
        self, value: List[ExtensionFulfillmentSummary]
    ) -> FulfillmentHistoryInfo:
        self.extension_fulfillment_summaries = value
        return self

    def with_fulfill_items(
        self, value: List[FulfillmentItem]
    ) -> FulfillmentHistoryInfo:
        self.fulfill_items = value
        return self

    def with_fulfillment_error(self, value: FulfillmentError) -> FulfillmentHistoryInfo:
        self.fulfillment_error = value
        return self

    def with_granted_item_ids(self, value: List[str]) -> FulfillmentHistoryInfo:
        self.granted_item_ids = value
        return self

    def with_order_no(self, value: str) -> FulfillmentHistoryInfo:
        self.order_no = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "code"):
            result["code"] = str(self.code)
        elif include_empty:
            result["code"] = ""
        if hasattr(self, "credit_summaries"):
            result["creditSummaries"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.credit_summaries
            ]
        elif include_empty:
            result["creditSummaries"] = []
        if hasattr(self, "entitlement_origin"):
            result["entitlementOrigin"] = str(self.entitlement_origin)
        elif include_empty:
            result["entitlementOrigin"] = Union[str, EntitlementOriginEnum]()
        if hasattr(self, "entitlement_summaries"):
            result["entitlementSummaries"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.entitlement_summaries
            ]
        elif include_empty:
            result["entitlementSummaries"] = []
        if hasattr(self, "extension_fulfillment_summaries"):
            result["extensionFulfillmentSummaries"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.extension_fulfillment_summaries
            ]
        elif include_empty:
            result["extensionFulfillmentSummaries"] = []
        if hasattr(self, "fulfill_items"):
            result["fulfillItems"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.fulfill_items
            ]
        elif include_empty:
            result["fulfillItems"] = []
        if hasattr(self, "fulfillment_error"):
            result["fulfillmentError"] = self.fulfillment_error.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["fulfillmentError"] = FulfillmentError()
        if hasattr(self, "granted_item_ids"):
            result["grantedItemIds"] = [str(i0) for i0 in self.granted_item_ids]
        elif include_empty:
            result["grantedItemIds"] = []
        if hasattr(self, "order_no"):
            result["orderNo"] = str(self.order_no)
        elif include_empty:
            result["orderNo"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        created_at: str,
        id_: str,
        namespace: str,
        status: Union[str, StatusEnum],
        updated_at: str,
        user_id: str,
        code: Optional[str] = None,
        credit_summaries: Optional[List[CreditSummary]] = None,
        entitlement_origin: Optional[Union[str, EntitlementOriginEnum]] = None,
        entitlement_summaries: Optional[List[EntitlementSummary]] = None,
        extension_fulfillment_summaries: Optional[
            List[ExtensionFulfillmentSummary]
        ] = None,
        fulfill_items: Optional[List[FulfillmentItem]] = None,
        fulfillment_error: Optional[FulfillmentError] = None,
        granted_item_ids: Optional[List[str]] = None,
        order_no: Optional[str] = None,
        **kwargs,
    ) -> FulfillmentHistoryInfo:
        instance = cls()
        instance.created_at = created_at
        instance.id_ = id_
        instance.namespace = namespace
        instance.status = status
        instance.updated_at = updated_at
        instance.user_id = user_id
        if code is not None:
            instance.code = code
        if credit_summaries is not None:
            instance.credit_summaries = credit_summaries
        if entitlement_origin is not None:
            instance.entitlement_origin = entitlement_origin
        if entitlement_summaries is not None:
            instance.entitlement_summaries = entitlement_summaries
        if extension_fulfillment_summaries is not None:
            instance.extension_fulfillment_summaries = extension_fulfillment_summaries
        if fulfill_items is not None:
            instance.fulfill_items = fulfill_items
        if fulfillment_error is not None:
            instance.fulfillment_error = fulfillment_error
        if granted_item_ids is not None:
            instance.granted_item_ids = granted_item_ids
        if order_no is not None:
            instance.order_no = order_no
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> FulfillmentHistoryInfo:
        instance = cls()
        if not dict_:
            return instance
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "code" in dict_ and dict_["code"] is not None:
            instance.code = str(dict_["code"])
        elif include_empty:
            instance.code = ""
        if "creditSummaries" in dict_ and dict_["creditSummaries"] is not None:
            instance.credit_summaries = [
                CreditSummary.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["creditSummaries"]
            ]
        elif include_empty:
            instance.credit_summaries = []
        if "entitlementOrigin" in dict_ and dict_["entitlementOrigin"] is not None:
            instance.entitlement_origin = str(dict_["entitlementOrigin"])
        elif include_empty:
            instance.entitlement_origin = Union[str, EntitlementOriginEnum]()
        if (
            "entitlementSummaries" in dict_
            and dict_["entitlementSummaries"] is not None
        ):
            instance.entitlement_summaries = [
                EntitlementSummary.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["entitlementSummaries"]
            ]
        elif include_empty:
            instance.entitlement_summaries = []
        if (
            "extensionFulfillmentSummaries" in dict_
            and dict_["extensionFulfillmentSummaries"] is not None
        ):
            instance.extension_fulfillment_summaries = [
                ExtensionFulfillmentSummary.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["extensionFulfillmentSummaries"]
            ]
        elif include_empty:
            instance.extension_fulfillment_summaries = []
        if "fulfillItems" in dict_ and dict_["fulfillItems"] is not None:
            instance.fulfill_items = [
                FulfillmentItem.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["fulfillItems"]
            ]
        elif include_empty:
            instance.fulfill_items = []
        if "fulfillmentError" in dict_ and dict_["fulfillmentError"] is not None:
            instance.fulfillment_error = FulfillmentError.create_from_dict(
                dict_["fulfillmentError"], include_empty=include_empty
            )
        elif include_empty:
            instance.fulfillment_error = FulfillmentError()
        if "grantedItemIds" in dict_ and dict_["grantedItemIds"] is not None:
            instance.granted_item_ids = [str(i0) for i0 in dict_["grantedItemIds"]]
        elif include_empty:
            instance.granted_item_ids = []
        if "orderNo" in dict_ and dict_["orderNo"] is not None:
            instance.order_no = str(dict_["orderNo"])
        elif include_empty:
            instance.order_no = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, FulfillmentHistoryInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[FulfillmentHistoryInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        FulfillmentHistoryInfo,
        List[FulfillmentHistoryInfo],
        Dict[Any, FulfillmentHistoryInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "createdAt": "created_at",
            "id": "id_",
            "namespace": "namespace",
            "status": "status",
            "updatedAt": "updated_at",
            "userId": "user_id",
            "code": "code",
            "creditSummaries": "credit_summaries",
            "entitlementOrigin": "entitlement_origin",
            "entitlementSummaries": "entitlement_summaries",
            "extensionFulfillmentSummaries": "extension_fulfillment_summaries",
            "fulfillItems": "fulfill_items",
            "fulfillmentError": "fulfillment_error",
            "grantedItemIds": "granted_item_ids",
            "orderNo": "order_no",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "createdAt": True,
            "id": True,
            "namespace": True,
            "status": True,
            "updatedAt": True,
            "userId": True,
            "code": False,
            "creditSummaries": False,
            "entitlementOrigin": False,
            "entitlementSummaries": False,
            "extensionFulfillmentSummaries": False,
            "fulfillItems": False,
            "fulfillmentError": False,
            "grantedItemIds": False,
            "orderNo": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["FAIL", "SUCCESS"],
            "entitlementOrigin": [
                "Epic",
                "GooglePlay",
                "IOS",
                "Nintendo",
                "Oculus",
                "Other",
                "Playstation",
                "Steam",
                "System",
                "Twitch",
                "Xbox",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum


class ActionEnum(StrEnum):
    FULFILLMENT = "FULFILLMENT"
    REVOKE_FULFILLMENT = "REVOKE_FULFILLMENT"


class StateEnum(StrEnum):
    FULFILLED = "FULFILLED"
    FULFILL_FAILED = "FULFILL_FAILED"
    REVOKED = "REVOKED"
    REVOKE_FAILED = "REVOKE_FAILED"


class FulfillmentActionHistory(Model):
    """Fulfillment action history (FulfillmentActionHistory)

    Properties:
        action: (action) OPTIONAL Union[str, ActionEnum]

        created_at: (createdAt) OPTIONAL str

        fulfillment_history_id: (fulfillmentHistoryId) OPTIONAL str

        revocation_history_id: (revocationHistoryId) OPTIONAL str

        state: (state) OPTIONAL Union[str, StateEnum]
    """

    # region fields

    action: Union[str, ActionEnum]  # OPTIONAL
    created_at: str  # OPTIONAL
    fulfillment_history_id: str  # OPTIONAL
    revocation_history_id: str  # OPTIONAL
    state: Union[str, StateEnum]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_action(self, value: Union[str, ActionEnum]) -> FulfillmentActionHistory:
        self.action = value
        return self

    def with_created_at(self, value: str) -> FulfillmentActionHistory:
        self.created_at = value
        return self

    def with_fulfillment_history_id(self, value: str) -> FulfillmentActionHistory:
        self.fulfillment_history_id = value
        return self

    def with_revocation_history_id(self, value: str) -> FulfillmentActionHistory:
        self.revocation_history_id = value
        return self

    def with_state(self, value: Union[str, StateEnum]) -> FulfillmentActionHistory:
        self.state = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "action"):
            result["action"] = str(self.action)
        elif include_empty:
            result["action"] = Union[str, ActionEnum]()
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "fulfillment_history_id"):
            result["fulfillmentHistoryId"] = str(self.fulfillment_history_id)
        elif include_empty:
            result["fulfillmentHistoryId"] = ""
        if hasattr(self, "revocation_history_id"):
            result["revocationHistoryId"] = str(self.revocation_history_id)
        elif include_empty:
            result["revocationHistoryId"] = ""
        if hasattr(self, "state"):
            result["state"] = str(self.state)
        elif include_empty:
            result["state"] = Union[str, StateEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        action: Optional[Union[str, ActionEnum]] = None,
        created_at: Optional[str] = None,
        fulfillment_history_id: Optional[str] = None,
        revocation_history_id: Optional[str] = None,
        state: Optional[Union[str, StateEnum]] = None,
        **kwargs,
    ) -> FulfillmentActionHistory:
        instance = cls()
        if action is not None:
            instance.action = action
        if created_at is not None:
            instance.created_at = created_at
        if fulfillment_history_id is not None:
            instance.fulfillment_history_id = fulfillment_history_id
        if revocation_history_id is not None:
            instance.revocation_history_id = revocation_history_id
        if state is not None:
            instance.state = state
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> FulfillmentActionHistory:
        instance = cls()
        if not dict_:
            return instance
        if "action" in dict_ and dict_["action"] is not None:
            instance.action = str(dict_["action"])
        elif include_empty:
            instance.action = Union[str, ActionEnum]()
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if (
            "fulfillmentHistoryId" in dict_
            and dict_["fulfillmentHistoryId"] is not None
        ):
            instance.fulfillment_history_id = str(dict_["fulfillmentHistoryId"])
        elif include_empty:
            instance.fulfillment_history_id = ""
        if "revocationHistoryId" in dict_ and dict_["revocationHistoryId"] is not None:
            instance.revocation_history_id = str(dict_["revocationHistoryId"])
        elif include_empty:
            instance.revocation_history_id = ""
        if "state" in dict_ and dict_["state"] is not None:
            instance.state = str(dict_["state"])
        elif include_empty:
            instance.state = Union[str, StateEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, FulfillmentActionHistory]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[FulfillmentActionHistory]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        FulfillmentActionHistory,
        List[FulfillmentActionHistory],
        Dict[Any, FulfillmentActionHistory],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "action": "action",
            "createdAt": "created_at",
            "fulfillmentHistoryId": "fulfillment_history_id",
            "revocationHistoryId": "revocation_history_id",
            "state": "state",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "action": False,
            "createdAt": False,
            "fulfillmentHistoryId": False,
            "revocationHistoryId": False,
            "state": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "action": ["FULFILLMENT", "REVOKE_FULFILLMENT"],
            "state": ["FULFILLED", "FULFILL_FAILED", "REVOKED", "REVOKE_FAILED"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum


class NotifyTypeEnum(StrEnum):
    CHARGE = "CHARGE"
    REFUND = "REFUND"


class PaymentProviderEnum(StrEnum):
    ADYEN = "ADYEN"
    ALIPAY = "ALIPAY"
    CHECKOUT = "CHECKOUT"
    NEONPAY = "NEONPAY"
    PAYPAL = "PAYPAL"
    STRIPE = "STRIPE"
    WALLET = "WALLET"
    WXPAY = "WXPAY"
    XSOLLA = "XSOLLA"


class PaymentOrderNotifySimulation(Model):
    """Payment order notify simulation (PaymentOrderNotifySimulation)

    Properties:
        currency_code: (currencyCode) REQUIRED str

        notify_type: (notifyType) REQUIRED Union[str, NotifyTypeEnum]

        payment_provider: (paymentProvider) REQUIRED Union[str, PaymentProviderEnum]

        amount: (amount) OPTIONAL int

        sales_tax: (salesTax) OPTIONAL int

        vat: (vat) OPTIONAL int
    """

    # region fields

    currency_code: str  # REQUIRED
    notify_type: Union[str, NotifyTypeEnum]  # REQUIRED
    payment_provider: Union[str, PaymentProviderEnum]  # REQUIRED
    amount: int  # OPTIONAL
    sales_tax: int  # OPTIONAL
    vat: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_currency_code(self, value: str) -> PaymentOrderNotifySimulation:
        self.currency_code = value
        return self

    def with_notify_type(
        self, value: Union[str, NotifyTypeEnum]
    ) -> PaymentOrderNotifySimulation:
        self.notify_type = value
        return self

    def with_payment_provider(
        self, value: Union[str, PaymentProviderEnum]
    ) -> PaymentOrderNotifySimulation:
        self.payment_provider = value
        return self

    def with_amount(self, value: int) -> PaymentOrderNotifySimulation:
        self.amount = value
        return self

    def with_sales_tax(self, value: int) -> PaymentOrderNotifySimulation:
        self.sales_tax = value
        return self

    def with_vat(self, value: int) -> PaymentOrderNotifySimulation:
        self.vat = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "currency_code"):
            result["currencyCode"] = str(self.currency_code)
        elif include_empty:
            result["currencyCode"] = ""
        if hasattr(self, "notify_type"):
            result["notifyType"] = str(self.notify_type)
        elif include_empty:
            result["notifyType"] = Union[str, NotifyTypeEnum]()
        if hasattr(self, "payment_provider"):
            result["paymentProvider"] = str(self.payment_provider)
        elif include_empty:
            result["paymentProvider"] = Union[str, PaymentProviderEnum]()
        if hasattr(self, "amount"):
            result["amount"] = int(self.amount)
        elif include_empty:
            result["amount"] = 0
        if hasattr(self, "sales_tax"):
            result["salesTax"] = int(self.sales_tax)
        elif include_empty:
            result["salesTax"] = 0
        if hasattr(self, "vat"):
            result["vat"] = int(self.vat)
        elif include_empty:
            result["vat"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        currency_code: str,
        notify_type: Union[str, NotifyTypeEnum],
        payment_provider: Union[str, PaymentProviderEnum],
        amount: Optional[int] = None,
        sales_tax: Optional[int] = None,
        vat: Optional[int] = None,
        **kwargs,
    ) -> PaymentOrderNotifySimulation:
        instance = cls()
        instance.currency_code = currency_code
        instance.notify_type = notify_type
        instance.payment_provider = payment_provider
        if amount is not None:
            instance.amount = amount
        if sales_tax is not None:
            instance.sales_tax = sales_tax
        if vat is not None:
            instance.vat = vat
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> PaymentOrderNotifySimulation:
        instance = cls()
        if not dict_:
            return instance
        if "currencyCode" in dict_ and dict_["currencyCode"] is not None:
            instance.currency_code = str(dict_["currencyCode"])
        elif include_empty:
            instance.currency_code = ""
        if "notifyType" in dict_ and dict_["notifyType"] is not None:
            instance.notify_type = str(dict_["notifyType"])
        elif include_empty:
            instance.notify_type = Union[str, NotifyTypeEnum]()
        if "paymentProvider" in dict_ and dict_["paymentProvider"] is not None:
            instance.payment_provider = str(dict_["paymentProvider"])
        elif include_empty:
            instance.payment_provider = Union[str, PaymentProviderEnum]()
        if "amount" in dict_ and dict_["amount"] is not None:
            instance.amount = int(dict_["amount"])
        elif include_empty:
            instance.amount = 0
        if "salesTax" in dict_ and dict_["salesTax"] is not None:
            instance.sales_tax = int(dict_["salesTax"])
        elif include_empty:
            instance.sales_tax = 0
        if "vat" in dict_ and dict_["vat"] is not None:
            instance.vat = int(dict_["vat"])
        elif include_empty:
            instance.vat = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, PaymentOrderNotifySimulation]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[PaymentOrderNotifySimulation]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        PaymentOrderNotifySimulation,
        List[PaymentOrderNotifySimulation],
        Dict[Any, PaymentOrderNotifySimulation],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "currencyCode": "currency_code",
            "notifyType": "notify_type",
            "paymentProvider": "payment_provider",
            "amount": "amount",
            "salesTax": "sales_tax",
            "vat": "vat",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "currencyCode": True,
            "notifyType": True,
            "paymentProvider": True,
            "amount": False,
            "salesTax": False,
            "vat": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "notifyType": ["CHARGE", "REFUND"],
            "paymentProvider": [
                "ADYEN",
                "ALIPAY",
                "CHECKOUT",
                "NEONPAY",
                "PAYPAL",
                "STRIPE",
                "WALLET",
                "WXPAY",
                "XSOLLA",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum

from ..models.currency_summary import CurrencySummary
from ..models.deduction_detail import DeductionDetail
from ..models.item_snapshot import ItemSnapshot
from ..models.order_bundle_item_info import OrderBundleItemInfo
from ..models.order_creation_options import OrderCreationOptions
from ..models.payment_data import PaymentData


class PaymentProviderEnum(StrEnum):
    ADYEN = "ADYEN"
    ALIPAY = "ALIPAY"
    CHECKOUT = "CHECKOUT"
    NEONPAY = "NEONPAY"
    PAYPAL = "PAYPAL"
    STRIPE = "STRIPE"
    WALLET = "WALLET"
    WXPAY = "WXPAY"
    XSOLLA = "XSOLLA"


class StatusEnum(StrEnum):
    CHARGEBACK = "CHARGEBACK"
    CHARGEBACK_REVERSED = "CHARGEBACK_REVERSED"
    CHARGED = "CHARGED"
    CLOSED = "CLOSED"
    DELETED = "DELETED"
    FULFILLED = "FULFILLED"
    FULFILL_FAILED = "FULFILL_FAILED"
    INIT = "INIT"
    REFUNDED = "REFUNDED"
    REFUNDING = "REFUNDING"
    REFUND_FAILED = "REFUND_FAILED"


class Order(Model):
    """Order (Order)

    Properties:
        chargeback_reversed_time: (chargebackReversedTime) OPTIONAL str

        chargeback_time: (chargebackTime) OPTIONAL str

        charged: (charged) OPTIONAL bool

        charged_time: (chargedTime) OPTIONAL str

        count_item_id: (countItemId) OPTIONAL str

        count_namespace: (countNamespace) OPTIONAL str

        count_user_id: (countUserId) OPTIONAL str

        created_at: (createdAt) OPTIONAL str

        created_time: (createdTime) OPTIONAL str

        creation_options: (creationOptions) OPTIONAL OrderCreationOptions

        currency: (currency) OPTIONAL CurrencySummary

        deduction: (deduction) OPTIONAL int

        deduction_details: (deductionDetails) OPTIONAL List[DeductionDetail]

        discounted_price: (discountedPrice) OPTIONAL int

        expire_time: (expireTime) OPTIONAL str

        ext: (ext) OPTIONAL Dict[str, Any]

        final_price: (finalPrice) OPTIONAL int

        free: (free) OPTIONAL bool

        fulfilled_time: (fulfilledTime) OPTIONAL str

        item_id: (itemId) OPTIONAL str

        item_snapshot: (itemSnapshot) OPTIONAL ItemSnapshot

        language: (language) OPTIONAL str

        namespace: (namespace) OPTIONAL str

        order_bundle_item_infos: (orderBundleItemInfos) OPTIONAL List[OrderBundleItemInfo]

        order_no: (orderNo) OPTIONAL str

        payment_data: (paymentData) OPTIONAL PaymentData

        payment_method: (paymentMethod) OPTIONAL str

        payment_method_fee: (paymentMethodFee) OPTIONAL int

        payment_order_no: (paymentOrderNo) OPTIONAL str

        payment_provider: (paymentProvider) OPTIONAL Union[str, PaymentProviderEnum]

        payment_provider_fee: (paymentProviderFee) OPTIONAL int

        payment_remain_seconds: (paymentRemainSeconds) OPTIONAL int

        payment_station_url: (paymentStationUrl) OPTIONAL str

        price: (price) OPTIONAL int

        quantity: (quantity) OPTIONAL int

        refunded_time: (refundedTime) OPTIONAL str

        region: (region) OPTIONAL str

        return_url: (returnUrl) OPTIONAL str

        rvn: (rvn) OPTIONAL int

        sales_tax: (salesTax) OPTIONAL int

        sandbox: (sandbox) OPTIONAL bool

        status: (status) OPTIONAL Union[str, StatusEnum]

        status_reason: (statusReason) OPTIONAL str

        subtotal_price: (subtotalPrice) OPTIONAL int

        tax: (tax) OPTIONAL int

        total_price: (totalPrice) OPTIONAL int

        total_tax: (totalTax) OPTIONAL int

        updated_at: (updatedAt) OPTIONAL str

        user_id: (userId) OPTIONAL str

        vat: (vat) OPTIONAL int
    """

    # region fields

    chargeback_reversed_time: str  # OPTIONAL
    chargeback_time: str  # OPTIONAL
    charged: bool  # OPTIONAL
    charged_time: str  # OPTIONAL
    count_item_id: str  # OPTIONAL
    count_namespace: str  # OPTIONAL
    count_user_id: str  # OPTIONAL
    created_at: str  # OPTIONAL
    created_time: str  # OPTIONAL
    creation_options: OrderCreationOptions  # OPTIONAL
    currency: CurrencySummary  # OPTIONAL
    deduction: int  # OPTIONAL
    deduction_details: List[DeductionDetail]  # OPTIONAL
    discounted_price: int  # OPTIONAL
    expire_time: str  # OPTIONAL
    ext: Dict[str, Any]  # OPTIONAL
    final_price: int  # OPTIONAL
    free: bool  # OPTIONAL
    fulfilled_time: str  # OPTIONAL
    item_id: str  # OPTIONAL
    item_snapshot: ItemSnapshot  # OPTIONAL
    language: str  # OPTIONAL
    namespace: str  # OPTIONAL
    order_bundle_item_infos: List[OrderBundleItemInfo]  # OPTIONAL
    order_no: str  # OPTIONAL
    payment_data: PaymentData  # OPTIONAL
    payment_method: str  # OPTIONAL
    payment_method_fee: int  # OPTIONAL
    payment_order_no: str  # OPTIONAL
    payment_provider: Union[str, PaymentProviderEnum]  # OPTIONAL
    payment_provider_fee: int  # OPTIONAL
    payment_remain_seconds: int  # OPTIONAL
    payment_station_url: str  # OPTIONAL
    price: int  # OPTIONAL
    quantity: int  # OPTIONAL
    refunded_time: str  # OPTIONAL
    region: str  # OPTIONAL
    return_url: str  # OPTIONAL
    rvn: int  # OPTIONAL
    sales_tax: int  # OPTIONAL
    sandbox: bool  # OPTIONAL
    status: Union[str, StatusEnum]  # OPTIONAL
    status_reason: str  # OPTIONAL
    subtotal_price: int  # OPTIONAL
    tax: int  # OPTIONAL
    total_price: int  # OPTIONAL
    total_tax: int  # OPTIONAL
    updated_at: str  # OPTIONAL
    user_id: str  # OPTIONAL
    vat: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_chargeback_reversed_time(self, value: str) -> Order:
        self.chargeback_reversed_time = value
        return self

    def with_chargeback_time(self, value: str) -> Order:
        self.chargeback_time = value
        return self

    def with_charged(self, value: bool) -> Order:
        self.charged = value
        return self

    def with_charged_time(self, value: str) -> Order:
        self.charged_time = value
        return self

    def with_count_item_id(self, value: str) -> Order:
        self.count_item_id = value
        return self

    def with_count_namespace(self, value: str) -> Order:
        self.count_namespace = value
        return self

    def with_count_user_id(self, value: str) -> Order:
        self.count_user_id = value
        return self

    def with_created_at(self, value: str) -> Order:
        self.created_at = value
        return self

    def with_created_time(self, value: str) -> Order:
        self.created_time = value
        return self

    def with_creation_options(self, value: OrderCreationOptions) -> Order:
        self.creation_options = value
        return self

    def with_currency(self, value: CurrencySummary) -> Order:
        self.currency = value
        return self

    def with_deduction(self, value: int) -> Order:
        self.deduction = value
        return self

    def with_deduction_details(self, value: List[DeductionDetail]) -> Order:
        self.deduction_details = value
        return self

    def with_discounted_price(self, value: int) -> Order:
        self.discounted_price = value
        return self

    def with_expire_time(self, value: str) -> Order:
        self.expire_time = value
        return self

    def with_ext(self, value: Dict[str, Any]) -> Order:
        self.ext = value
        return self

    def with_final_price(self, value: int) -> Order:
        self.final_price = value
        return self

    def with_free(self, value: bool) -> Order:
        self.free = value
        return self

    def with_fulfilled_time(self, value: str) -> Order:
        self.fulfilled_time = value
        return self

    def with_item_id(self, value: str) -> Order:
        self.item_id = value
        return self

    def with_item_snapshot(self, value: ItemSnapshot) -> Order:
        self.item_snapshot = value
        return self

    def with_language(self, value: str) -> Order:
        self.language = value
        return self

    def with_namespace(self, value: str) -> Order:
        self.namespace = value
        return self

    def with_order_bundle_item_infos(self, value: List[OrderBundleItemInfo]) -> Order:
        self.order_bundle_item_infos = value
        return self

    def with_order_no(self, value: str) -> Order:
        self.order_no = value
        return self

    def with_payment_data(self, value: PaymentData) -> Order:
        self.payment_data = value
        return self

    def with_payment_method(self, value: str) -> Order:
        self.payment_method = value
        return self

    def with_payment_method_fee(self, value: int) -> Order:
        self.payment_method_fee = value
        return self

    def with_payment_order_no(self, value: str) -> Order:
        self.payment_order_no = value
        return self

    def with_payment_provider(self, value: Union[str, PaymentProviderEnum]) -> Order:
        self.payment_provider = value
        return self

    def with_payment_provider_fee(self, value: int) -> Order:
        self.payment_provider_fee = value
        return self

    def with_payment_remain_seconds(self, value: int) -> Order:
        self.payment_remain_seconds = value
        return self

    def with_payment_station_url(self, value: str) -> Order:
        self.payment_station_url = value
        return self

    def with_price(self, value: int) -> Order:
        self.price = value
        return self

    def with_quantity(self, value: int) -> Order:
        self.quantity = value
        return self

    def with_refunded_time(self, value: str) -> Order:
        self.refunded_time = value
        return self

    def with_region(self, value: str) -> Order:
        self.region = value
        return self

    def with_return_url(self, value: str) -> Order:
        self.return_url = value
        return self

    def with_rvn(self, value: int) -> Order:
        self.rvn = value
        return self

    def with_sales_tax(self, value: int) -> Order:
        self.sales_tax = value
        return self

    def with_sandbox(self, value: bool) -> Order:
        self.sandbox = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> Order:
        self.status = value
        return self

    def with_status_reason(self, value: str) -> Order:
        self.status_reason = value
        return self

    def with_subtotal_price(self, value: int) -> Order:
        self.subtotal_price = value
        return self

    def with_tax(self, value: int) -> Order:
        self.tax = value
        return self

    def with_total_price(self, value: int) -> Order:
        self.total_price = value
        return self

    def with_total_tax(self, value: int) -> Order:
        self.total_tax = value
        return self

    def with_updated_at(self, value: str) -> Order:
        self.updated_at = value
        return self

    def with_user_id(self, value: str) -> Order:
        self.user_id = value
        return self

    def with_vat(self, value: int) -> Order:
        self.vat = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "chargeback_reversed_time"):
            result["chargebackReversedTime"] = str(self.chargeback_reversed_time)
        elif include_empty:
            result["chargebackReversedTime"] = ""
        if hasattr(self, "chargeback_time"):
            result["chargebackTime"] = str(self.chargeback_time)
        elif include_empty:
            result["chargebackTime"] = ""
        if hasattr(self, "charged"):
            result["charged"] = bool(self.charged)
        elif include_empty:
            result["charged"] = False
        if hasattr(self, "charged_time"):
            result["chargedTime"] = str(self.charged_time)
        elif include_empty:
            result["chargedTime"] = ""
        if hasattr(self, "count_item_id"):
            result["countItemId"] = str(self.count_item_id)
        elif include_empty:
            result["countItemId"] = ""
        if hasattr(self, "count_namespace"):
            result["countNamespace"] = str(self.count_namespace)
        elif include_empty:
            result["countNamespace"] = ""
        if hasattr(self, "count_user_id"):
            result["countUserId"] = str(self.count_user_id)
        elif include_empty:
            result["countUserId"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "created_time"):
            result["createdTime"] = str(self.created_time)
        elif include_empty:
            result["createdTime"] = ""
        if hasattr(self, "creation_options"):
            result["creationOptions"] = self.creation_options.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["creationOptions"] = OrderCreationOptions()
        if hasattr(self, "currency"):
            result["currency"] = self.currency.to_dict(include_empty=include_empty)
        elif include_empty:
            result["currency"] = CurrencySummary()
        if hasattr(self, "deduction"):
            result["deduction"] = int(self.deduction)
        elif include_empty:
            result["deduction"] = 0
        if hasattr(self, "deduction_details"):
            result["deductionDetails"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.deduction_details
            ]
        elif include_empty:
            result["deductionDetails"] = []
        if hasattr(self, "discounted_price"):
            result["discountedPrice"] = int(self.discounted_price)
        elif include_empty:
            result["discountedPrice"] = 0
        if hasattr(self, "expire_time"):
            result["expireTime"] = str(self.expire_time)
        elif include_empty:
            result["expireTime"] = ""
        if hasattr(self, "ext"):
            result["ext"] = {str(k0): v0 for k0, v0 in self.ext.items()}
        elif include_empty:
            result["ext"] = {}
        if hasattr(self, "final_price"):
            result["finalPrice"] = int(self.final_price)
        elif include_empty:
            result["finalPrice"] = 0
        if hasattr(self, "free"):
            result["free"] = bool(self.free)
        elif include_empty:
            result["free"] = False
        if hasattr(self, "fulfilled_time"):
            result["fulfilledTime"] = str(self.fulfilled_time)
        elif include_empty:
            result["fulfilledTime"] = ""
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "item_snapshot"):
            result["itemSnapshot"] = self.item_snapshot.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["itemSnapshot"] = ItemSnapshot()
        if hasattr(self, "language"):
            result["language"] = str(self.language)
        elif include_empty:
            result["language"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "order_bundle_item_infos"):
            result["orderBundleItemInfos"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.order_bundle_item_infos
            ]
        elif include_empty:
            result["orderBundleItemInfos"] = []
        if hasattr(self, "order_no"):
            result["orderNo"] = str(self.order_no)
        elif include_empty:
            result["orderNo"] = ""
        if hasattr(self, "payment_data"):
            result["paymentData"] = self.payment_data.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["paymentData"] = PaymentData()
        if hasattr(self, "payment_method"):
            result["paymentMethod"] = str(self.payment_method)
        elif include_empty:
            result["paymentMethod"] = ""
        if hasattr(self, "payment_method_fee"):
            result["paymentMethodFee"] = int(self.payment_method_fee)
        elif include_empty:
            result["paymentMethodFee"] = 0
        if hasattr(self, "payment_order_no"):
            result["paymentOrderNo"] = str(self.payment_order_no)
        elif include_empty:
            result["paymentOrderNo"] = ""
        if hasattr(self, "payment_provider"):
            result["paymentProvider"] = str(self.payment_provider)
        elif include_empty:
            result["paymentProvider"] = Union[str, PaymentProviderEnum]()
        if hasattr(self, "payment_provider_fee"):
            result["paymentProviderFee"] = int(self.payment_provider_fee)
        elif include_empty:
            result["paymentProviderFee"] = 0
        if hasattr(self, "payment_remain_seconds"):
            result["paymentRemainSeconds"] = int(self.payment_remain_seconds)
        elif include_empty:
            result["paymentRemainSeconds"] = 0
        if hasattr(self, "payment_station_url"):
            result["paymentStationUrl"] = str(self.payment_station_url)
        elif include_empty:
            result["paymentStationUrl"] = ""
        if hasattr(self, "price"):
            result["price"] = int(self.price)
        elif include_empty:
            result["price"] = 0
        if hasattr(self, "quantity"):
            result["quantity"] = int(self.quantity)
        elif include_empty:
            result["quantity"] = 0
        if hasattr(self, "refunded_time"):
            result["refundedTime"] = str(self.refunded_time)
        elif include_empty:
            result["refundedTime"] = ""
        if hasattr(self, "region"):
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "return_url"):
            result["returnUrl"] = str(self.return_url)
        elif include_empty:
            result["returnUrl"] = ""
        if hasattr(self, "rvn"):
            result["rvn"] = int(self.rvn)
        elif include_empty:
            result["rvn"] = 0
        if hasattr(self, "sales_tax"):
            result["salesTax"] = int(self.sales_tax)
        elif include_empty:
            result["salesTax"] = 0
        if hasattr(self, "sandbox"):
            result["sandbox"] = bool(self.sandbox)
        elif include_empty:
            result["sandbox"] = False
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "status_reason"):
            result["statusReason"] = str(self.status_reason)
        elif include_empty:
            result["statusReason"] = ""
        if hasattr(self, "subtotal_price"):
            result["subtotalPrice"] = int(self.subtotal_price)
        elif include_empty:
            result["subtotalPrice"] = 0
        if hasattr(self, "tax"):
            result["tax"] = int(self.tax)
        elif include_empty:
            result["tax"] = 0
        if hasattr(self, "total_price"):
            result["totalPrice"] = int(self.total_price)
        elif include_empty:
            result["totalPrice"] = 0
        if hasattr(self, "total_tax"):
            result["totalTax"] = int(self.total_tax)
        elif include_empty:
            result["totalTax"] = 0
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "vat"):
            result["vat"] = int(self.vat)
        elif include_empty:
            result["vat"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        chargeback_reversed_time: Optional[str] = None,
        chargeback_time: Optional[str] = None,
        charged: Optional[bool] = None,
        charged_time: Optional[str] = None,
        count_item_id: Optional[str] = None,
        count_namespace: Optional[str] = None,
        count_user_id: Optional[str] = None,
        created_at: Optional[str] = None,
        created_time: Optional[str] = None,
        creation_options: Optional[OrderCreationOptions] = None,
        currency: Optional[CurrencySummary] = None,
        deduction: Optional[int] = None,
        deduction_details: Optional[List[DeductionDetail]] = None,
        discounted_price: Optional[int] = None,
        expire_time: Optional[str] = None,
        ext: Optional[Dict[str, Any]] = None,
        final_price: Optional[int] = None,
        free: Optional[bool] = None,
        fulfilled_time: Optional[str] = None,
        item_id: Optional[str] = None,
        item_snapshot: Optional[ItemSnapshot] = None,
        language: Optional[str] = None,
        namespace: Optional[str] = None,
        order_bundle_item_infos: Optional[List[OrderBundleItemInfo]] = None,
        order_no: Optional[str] = None,
        payment_data: Optional[PaymentData] = None,
        payment_method: Optional[str] = None,
        payment_method_fee: Optional[int] = None,
        payment_order_no: Optional[str] = None,
        payment_provider: Optional[Union[str, PaymentProviderEnum]] = None,
        payment_provider_fee: Optional[int] = None,
        payment_remain_seconds: Optional[int] = None,
        payment_station_url: Optional[str] = None,
        price: Optional[int] = None,
        quantity: Optional[int] = None,
        refunded_time: Optional[str] = None,
        region: Optional[str] = None,
        return_url: Optional[str] = None,
        rvn: Optional[int] = None,
        sales_tax: Optional[int] = None,
        sandbox: Optional[bool] = None,
        status: Optional[Union[str, StatusEnum]] = None,
        status_reason: Optional[str] = None,
        subtotal_price: Optional[int] = None,
        tax: Optional[int] = None,
        total_price: Optional[int] = None,
        total_tax: Optional[int] = None,
        updated_at: Optional[str] = None,
        user_id: Optional[str] = None,
        vat: Optional[int] = None,
        **kwargs,
    ) -> Order:
        instance = cls()
        if chargeback_reversed_time is not None:
            instance.chargeback_reversed_time = chargeback_reversed_time
        if chargeback_time is not None:
            instance.chargeback_time = chargeback_time
        if charged is not None:
            instance.charged = charged
        if charged_time is not None:
            instance.charged_time = charged_time
        if count_item_id is not None:
            instance.count_item_id = count_item_id
        if count_namespace is not None:
            instance.count_namespace = count_namespace
        if count_user_id is not None:
            instance.count_user_id = count_user_id
        if created_at is not None:
            instance.created_at = created_at
        if created_time is not None:
            instance.created_time = created_time
        if creation_options is not None:
            instance.creation_options = creation_options
        if currency is not None:
            instance.currency = currency
        if deduction is not None:
            instance.deduction = deduction
        if deduction_details is not None:
            instance.deduction_details = deduction_details
        if discounted_price is not None:
            instance.discounted_price = discounted_price
        if expire_time is not None:
            instance.expire_time = expire_time
        if ext is not None:
            instance.ext = ext
        if final_price is not None:
            instance.final_price = final_price
        if free is not None:
            instance.free = free
        if fulfilled_time is not None:
            instance.fulfilled_time = fulfilled_time
        if item_id is not None:
            instance.item_id = item_id
        if item_snapshot is not None:
            instance.item_snapshot = item_snapshot
        if language is not None:
            instance.language = language
        if namespace is not None:
            instance.namespace = namespace
        if order_bundle_item_infos is not None:
            instance.order_bundle_item_infos = order_bundle_item_infos
        if order_no is not None:
            instance.order_no = order_no
        if payment_data is not None:
            instance.payment_data = payment_data
        if payment_method is not None:
            instance.payment_method = payment_method
        if payment_method_fee is not None:
            instance.payment_method_fee = payment_method_fee
        if payment_order_no is not None:
            instance.payment_order_no = payment_order_no
        if payment_provider is not None:
            instance.payment_provider = payment_provider
        if payment_provider_fee is not None:
            instance.payment_provider_fee = payment_provider_fee
        if payment_remain_seconds is not None:
            instance.payment_remain_seconds = payment_remain_seconds
        if payment_station_url is not None:
            instance.payment_station_url = payment_station_url
        if price is not None:
            instance.price = price
        if quantity is not None:
            instance.quantity = quantity
        if refunded_time is not None:
            instance.refunded_time = refunded_time
        if region is not None:
            instance.region = region
        if return_url is not None:
            instance.return_url = return_url
        if rvn is not None:
            instance.rvn = rvn
        if sales_tax is not None:
            instance.sales_tax = sales_tax
        if sandbox is not None:
            instance.sandbox = sandbox
        if status is not None:
            instance.status = status
        if status_reason is not None:
            instance.status_reason = status_reason
        if subtotal_price is not None:
            instance.subtotal_price = subtotal_price
        if tax is not None:
            instance.tax = tax
        if total_price is not None:
            instance.total_price = total_price
        if total_tax is not None:
            instance.total_tax = total_tax
        if updated_at is not None:
            instance.updated_at = updated_at
        if user_id is not None:
            instance.user_id = user_id
        if vat is not None:
            instance.vat = vat
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> Order:
        instance = cls()
        if not dict_:
            return instance
        if (
            "chargebackReversedTime" in dict_
            and dict_["chargebackReversedTime"] is not None
        ):
            instance.chargeback_reversed_time = str(dict_["chargebackReversedTime"])
        elif include_empty:
            instance.chargeback_reversed_time = ""
        if "chargebackTime" in dict_ and dict_["chargebackTime"] is not None:
            instance.chargeback_time = str(dict_["chargebackTime"])
        elif include_empty:
            instance.chargeback_time = ""
        if "charged" in dict_ and dict_["charged"] is not None:
            instance.charged = bool(dict_["charged"])
        elif include_empty:
            instance.charged = False
        if "chargedTime" in dict_ and dict_["chargedTime"] is not None:
            instance.charged_time = str(dict_["chargedTime"])
        elif include_empty:
            instance.charged_time = ""
        if "countItemId" in dict_ and dict_["countItemId"] is not None:
            instance.count_item_id = str(dict_["countItemId"])
        elif include_empty:
            instance.count_item_id = ""
        if "countNamespace" in dict_ and dict_["countNamespace"] is not None:
            instance.count_namespace = str(dict_["countNamespace"])
        elif include_empty:
            instance.count_namespace = ""
        if "countUserId" in dict_ and dict_["countUserId"] is not None:
            instance.count_user_id = str(dict_["countUserId"])
        elif include_empty:
            instance.count_user_id = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "createdTime" in dict_ and dict_["createdTime"] is not None:
            instance.created_time = str(dict_["createdTime"])
        elif include_empty:
            instance.created_time = ""
        if "creationOptions" in dict_ and dict_["creationOptions"] is not None:
            instance.creation_options = OrderCreationOptions.create_from_dict(
                dict_["creationOptions"], include_empty=include_empty
            )
        elif include_empty:
            instance.creation_options = OrderCreationOptions()
        if "currency" in dict_ and dict_["currency"] is not None:
            instance.currency = CurrencySummary.create_from_dict(
                dict_["currency"], include_empty=include_empty
            )
        elif include_empty:
            instance.currency = CurrencySummary()
        if "deduction" in dict_ and dict_["deduction"] is not None:
            instance.deduction = int(dict_["deduction"])
        elif include_empty:
            instance.deduction = 0
        if "deductionDetails" in dict_ and dict_["deductionDetails"] is not None:
            instance.deduction_details = [
                DeductionDetail.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["deductionDetails"]
            ]
        elif include_empty:
            instance.deduction_details = []
        if "discountedPrice" in dict_ and dict_["discountedPrice"] is not None:
            instance.discounted_price = int(dict_["discountedPrice"])
        elif include_empty:
            instance.discounted_price = 0
        if "expireTime" in dict_ and dict_["expireTime"] is not None:
            instance.expire_time = str(dict_["expireTime"])
        elif include_empty:
            instance.expire_time = ""
        if "ext" in dict_ and dict_["ext"] is not None:
            instance.ext = {str(k0): v0 for k0, v0 in dict_["ext"].items()}
        elif include_empty:
            instance.ext = {}
        if "finalPrice" in dict_ and dict_["finalPrice"] is not None:
            instance.final_price = int(dict_["finalPrice"])
        elif include_empty:
            instance.final_price = 0
        if "free" in dict_ and dict_["free"] is not None:
            instance.free = bool(dict_["free"])
        elif include_empty:
            instance.free = False
        if "fulfilledTime" in dict_ and dict_["fulfilledTime"] is not None:
            instance.fulfilled_time = str(dict_["fulfilledTime"])
        elif include_empty:
            instance.fulfilled_time = ""
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "itemSnapshot" in dict_ and dict_["itemSnapshot"] is not None:
            instance.item_snapshot = ItemSnapshot.create_from_dict(
                dict_["itemSnapshot"], include_empty=include_empty
            )
        elif include_empty:
            instance.item_snapshot = ItemSnapshot()
        if "language" in dict_ and dict_["language"] is not None:
            instance.language = str(dict_["language"])
        elif include_empty:
            instance.language = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if (
            "orderBundleItemInfos" in dict_
            and dict_["orderBundleItemInfos"] is not None
        ):
            instance.order_bundle_item_infos = [
                OrderBundleItemInfo.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["orderBundleItemInfos"]
            ]
        elif include_empty:
            instance.order_bundle_item_infos = []
        if "orderNo" in dict_ and dict_["orderNo"] is not None:
            instance.order_no = str(dict_["orderNo"])
        elif include_empty:
            instance.order_no = ""
        if "paymentData" in dict_ and dict_["paymentData"] is not None:
            instance.payment_data = PaymentData.create_from_dict(
                dict_["paymentData"], include_empty=include_empty
            )
        elif include_empty:
            instance.payment_data = PaymentData()
        if "paymentMethod" in dict_ and dict_["paymentMethod"] is not None:
            instance.payment_method = str(dict_["paymentMethod"])
        elif include_empty:
            instance.payment_method = ""
        if "paymentMethodFee" in dict_ and dict_["paymentMethodFee"] is not None:
            instance.payment_method_fee = int(dict_["paymentMethodFee"])
        elif include_empty:
            instance.payment_method_fee = 0
        if "paymentOrderNo" in dict_ and dict_["paymentOrderNo"] is not None:
            instance.payment_order_no = str(dict_["paymentOrderNo"])
        elif include_empty:
            instance.payment_order_no = ""
        if "paymentProvider" in dict_ and dict_["paymentProvider"] is not None:
            instance.payment_provider = str(dict_["paymentProvider"])
        elif include_empty:
            instance.payment_provider = Union[str, PaymentProviderEnum]()
        if "paymentProviderFee" in dict_ and dict_["paymentProviderFee"] is not None:
            instance.payment_provider_fee = int(dict_["paymentProviderFee"])
        elif include_empty:
            instance.payment_provider_fee = 0
        if (
            "paymentRemainSeconds" in dict_
            and dict_["paymentRemainSeconds"] is not None
        ):
            instance.payment_remain_seconds = int(dict_["paymentRemainSeconds"])
        elif include_empty:
            instance.payment_remain_seconds = 0
        if "paymentStationUrl" in dict_ and dict_["paymentStationUrl"] is not None:
            instance.payment_station_url = str(dict_["paymentStationUrl"])
        elif include_empty:
            instance.payment_station_url = ""
        if "price" in dict_ and dict_["price"] is not None:
            instance.price = int(dict_["price"])
        elif include_empty:
            instance.price = 0
        if "quantity" in dict_ and dict_["quantity"] is not None:
            instance.quantity = int(dict_["quantity"])
        elif include_empty:
            instance.quantity = 0
        if "refundedTime" in dict_ and dict_["refundedTime"] is not None:
            instance.refunded_time = str(dict_["refundedTime"])
        elif include_empty:
            instance.refunded_time = ""
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "returnUrl" in dict_ and dict_["returnUrl"] is not None:
            instance.return_url = str(dict_["returnUrl"])
        elif include_empty:
            instance.return_url = ""
        if "rvn" in dict_ and dict_["rvn"] is not None:
            instance.rvn = int(dict_["rvn"])
        elif include_empty:
            instance.rvn = 0
        if "salesTax" in dict_ and dict_["salesTax"] is not None:
            instance.sales_tax = int(dict_["salesTax"])
        elif include_empty:
            instance.sales_tax = 0
        if "sandbox" in dict_ and dict_["sandbox"] is not None:
            instance.sandbox = bool(dict_["sandbox"])
        elif include_empty:
            instance.sandbox = False
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "statusReason" in dict_ and dict_["statusReason"] is not None:
            instance.status_reason = str(dict_["statusReason"])
        elif include_empty:
            instance.status_reason = ""
        if "subtotalPrice" in dict_ and dict_["subtotalPrice"] is not None:
            instance.subtotal_price = int(dict_["subtotalPrice"])
        elif include_empty:
            instance.subtotal_price = 0
        if "tax" in dict_ and dict_["tax"] is not None:
            instance.tax = int(dict_["tax"])
        elif include_empty:
            instance.tax = 0
        if "totalPrice" in dict_ and dict_["totalPrice"] is not None:
            instance.total_price = int(dict_["totalPrice"])
        elif include_empty:
            instance.total_price = 0
        if "totalTax" in dict_ and dict_["totalTax"] is not None:
            instance.total_tax = int(dict_["totalTax"])
        elif include_empty:
            instance.total_tax = 0
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "vat" in dict_ and dict_["vat"] is not None:
            instance.vat = int(dict_["vat"])
        elif include_empty:
            instance.vat = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, Order]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[Order]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[Order, List[Order], Dict[Any, Order]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "chargebackReversedTime": "chargeback_reversed_time",
            "chargebackTime": "chargeback_time",
            "charged": "charged",
            "chargedTime": "charged_time",
            "countItemId": "count_item_id",
            "countNamespace": "count_namespace",
            "countUserId": "count_user_id",
            "createdAt": "created_at",
            "createdTime": "created_time",
            "creationOptions": "creation_options",
            "currency": "currency",
            "deduction": "deduction",
            "deductionDetails": "deduction_details",
            "discountedPrice": "discounted_price",
            "expireTime": "expire_time",
            "ext": "ext",
            "finalPrice": "final_price",
            "free": "free",
            "fulfilledTime": "fulfilled_time",
            "itemId": "item_id",
            "itemSnapshot": "item_snapshot",
            "language": "language",
            "namespace": "namespace",
            "orderBundleItemInfos": "order_bundle_item_infos",
            "orderNo": "order_no",
            "paymentData": "payment_data",
            "paymentMethod": "payment_method",
            "paymentMethodFee": "payment_method_fee",
            "paymentOrderNo": "payment_order_no",
            "paymentProvider": "payment_provider",
            "paymentProviderFee": "payment_provider_fee",
            "paymentRemainSeconds": "payment_remain_seconds",
            "paymentStationUrl": "payment_station_url",
            "price": "price",
            "quantity": "quantity",
            "refundedTime": "refunded_time",
            "region": "region",
            "returnUrl": "return_url",
            "rvn": "rvn",
            "salesTax": "sales_tax",
            "sandbox": "sandbox",
            "status": "status",
            "statusReason": "status_reason",
            "subtotalPrice": "subtotal_price",
            "tax": "tax",
            "totalPrice": "total_price",
            "totalTax": "total_tax",
            "updatedAt": "updated_at",
            "userId": "user_id",
            "vat": "vat",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "chargebackReversedTime": False,
            "chargebackTime": False,
            "charged": False,
            "chargedTime": False,
            "countItemId": False,
            "countNamespace": False,
            "countUserId": False,
            "createdAt": False,
            "createdTime": False,
            "creationOptions": False,
            "currency": False,
            "deduction": False,
            "deductionDetails": False,
            "discountedPrice": False,
            "expireTime": False,
            "ext": False,
            "finalPrice": False,
            "free": False,
            "fulfilledTime": False,
            "itemId": False,
            "itemSnapshot": False,
            "language": False,
            "namespace": False,
            "orderBundleItemInfos": False,
            "orderNo": False,
            "paymentData": False,
            "paymentMethod": False,
            "paymentMethodFee": False,
            "paymentOrderNo": False,
            "paymentProvider": False,
            "paymentProviderFee": False,
            "paymentRemainSeconds": False,
            "paymentStationUrl": False,
            "price": False,
            "quantity": False,
            "refundedTime": False,
            "region": False,
            "returnUrl": False,
            "rvn": False,
            "salesTax": False,
            "sandbox": False,
            "status": False,
            "statusReason": False,
            "subtotalPrice": False,
            "tax": False,
            "totalPrice": False,
            "totalTax": False,
            "updatedAt": False,
            "userId": False,
            "vat": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "paymentProvider": [
                "ADYEN",
                "ALIPAY",
                "CHECKOUT",
                "NEONPAY",
                "PAYPAL",
                "STRIPE",
                "WALLET",
                "WXPAY",
                "XSOLLA",
            ],
            "status": [
                "CHARGEBACK",
                "CHARGEBACK_REVERSED",
                "CHARGED",
                "CLOSED",
                "DELETED",
                "FULFILLED",
                "FULFILL_FAILED",
                "INIT",
                "REFUNDED",
                "REFUNDING",
                "REFUND_FAILED",
            ],
        }

    # endregion static methods
